#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"fft.h"

typedef long INT32;
typedef unsigned short int INT16;
typedef unsigned char U_CHAR;

#define UCH(x)	((int) (x))
#define GET_2B(array,offset)  ((INT16) UCH(array[offset]) + \
			       (((INT16) UCH(array[offset+1])) << 8))
#define GET_4B(array,offset)  ((INT32) UCH(array[offset]) + \
			       (((INT32) UCH(array[offset+1])) << 8) + \
			       (((INT32) UCH(array[offset+2])) << 16) + \
			       (((INT32) UCH(array[offset+3])) << 24))
#define FREAD(file,buf,sizeofbuf)  \
  ((size_t) fread((void *) (buf), (size_t) 1, (size_t) (sizeofbuf), (file)))

int ReadDataSize(char *name);
void ReadImageData(char *name, U_CHAR *bmpfileheader, U_CHAR *bmpinfoheader, U_CHAR *color_table, U_CHAR *data);


int main()
{
   FILE *output_file = 0 ;
   
   U_CHAR bmpfileheader1[14] = { 0 } ;
   U_CHAR bmpinfoheader1[40] = { 0 } ;
   U_CHAR *data1, *data2, color_table1[1024];
   COMPLEX *c, *c2;

 
   INT32 biWidth = 0 ;		
   INT32 biHeight = 0 ;

   int i, j, k, temp;
   int histo_table[256] = { 0 };
   double max, min, scale, D, H;
   int D0=30, n=20;


   i = ReadDataSize("Fig4.11(a).bmp");
   data1 = (U_CHAR *)malloc( i );
   if (data1 == NULL) {
      exit(0);
   }
  
   data2 = (U_CHAR *)malloc( i );
   if (data2 == NULL) {
      exit(0);
   }

   ReadImageData("Fig4.11(a).bmp", bmpfileheader1, bmpinfoheader1, color_table1, data1);
   biWidth           =   GET_4B(bmpinfoheader1,4);
   biHeight          =   GET_4B(bmpinfoheader1,8);


   c = (COMPLEX *)malloc( biHeight*biWidth*sizeof(COMPLEX) );
   if (c == NULL) {
      exit(0);
   }
   c2 = (COMPLEX *)malloc( biHeight*biWidth*sizeof(COMPLEX) );
   if (c2 == NULL) {
      exit(0);
   }

   // Process the file
   for (i=0; i < biHeight; i++)
   {
       k = i* ((biWidth*1 +3)/4 *4);
       for (j=0; j < biWidth; j++)
       {
		   c[k].real = data1[k];
		   c[k].imag = 0.0;
		   if ((i+j)%2 == 1)
				c[k].real = 0.0-c[k].real;

           k = k+1;
       }
   }

   if (!FFT2D(c, biHeight, biWidth, 1))
   {
	   printf("Stop!\n");
	   exit(0);
   }

   // -------------------------

	//	  
	for (i=0; i < biHeight; i++)
    {
       k = i* ((biWidth*1 +3)/4 *4);
       for (j=0; j < biWidth; j++)
       {
		   D = sqrt( (i-biHeight/2)*(i-biHeight/2) + (j-biWidth/2)*(j-biWidth/2) );
		   H = 1/(1+pow(D/D0, 2*n));	
		   c[k].real = H*c[k].real;
		   c[k].imag = H*c[k].imag;

   		   //if ((i+j)%2 == 1)
			//	c[k].real = 0.0-c[k].real;


           k = k+1;
       }
   }
	


   // -----------------------------
   
   
   if (!FFT2D(c, biHeight, biWidth, -1))
   {
	   printf("Stop!\n");
	   exit(0);
   }
	
   //
   for (i=0; i < biHeight; i++)
   {
       k = i* ((biWidth*1 +3)/4 *4);
       for (j=0; j < biWidth; j++)
       {
		   if ((i+j)%2 == 1)
				c[k].real = 0.0-c[k].real;

           k = k+1;
       }
   }

   // find the maximum
   max = 0.0;
   min = 255.0;
   for (i=0; i < biHeight; i++)
   {
       k = i* ((biWidth*1 +3)/4 *4);
       for (j=0; j < biWidth; j++)
       {
		   if (c[k].real > max)
			   max = c[k].real;
		   if (c[k].real < min)
			   min = c[k].real;

           k = k+1;
       }
   }
   printf("max=%f\n", max);
   printf("min=%f\n", min);
   //
   scale = 255.0/(max-min);
   for (i=0; i < biHeight; i++)
   {
       k = i* ((biWidth*1 +3)/4 *4);
       for (j=0; j < biWidth; j++)
       {
		   temp = (int) ((c[k].real-min)*scale);
		   if (temp > 255)	temp = 255;
		   if (temp < 0) temp = 0;
		   data2[k] = (U_CHAR) temp;

           k = k+1;
       }
   }


   //
   /* }ҷsɮ */
   if( ( output_file = fopen("new4.bmp","wb") ) == NULL ){
      fprintf(stderr,"Output file can't open.\n");
      exit(0);
   }

   fwrite(bmpfileheader1, sizeof(bmpfileheader1), 1, output_file);
   fwrite(bmpinfoheader1, sizeof(bmpinfoheader1), 1, output_file);

   fwrite(color_table1, 1024, 1, output_file);
 
   fwrite(data2, ((biWidth*1 +3)/4 *4)*biHeight*1, 1, output_file);
 
   fclose (output_file);

   free(data1);
   free(data2);
   free(c);
   free(c2);

  
   return 0;
}

int ReadDataSize(char *name)
{
   FILE *input_file = 0 ;
   U_CHAR bmpfileheader[14] = { 0 } ;
   U_CHAR bmpinfoheader[40] = { 0 } ;
   
   INT32 biWidth = 0 ;		
   INT32 biHeight = 0 ;
   INT16 BitCount = 0 ;

   /* }ɮ */
   if( ( input_file = fopen(name,"rb") ) == NULL ){
      fprintf(stderr,"File can't open.\n");
      exit(0);
   }

   FREAD(input_file,bmpfileheader,14);
   FREAD(input_file,bmpinfoheader,40);

   if (GET_2B(bmpfileheader,0) == 0x4D42) /* 'BM' */
      fprintf(stdout,"BMP file.\n");
   else{
      fprintf(stdout,"Not bmp file.\n");
      exit(0);
   }

   biWidth           =   GET_4B(bmpinfoheader,4);
   biHeight          =   GET_4B(bmpinfoheader,8);
   BitCount          =   GET_2B(bmpinfoheader,14);

   if (BitCount != 8) {
      fprintf(stderr,"Not a 8-bit file.\n");
      fclose (input_file);
      exit(0);
   }

   // 
   fclose (input_file);

   return ((biWidth*1 +3)/4 *4)*biHeight*1;
}

void ReadImageData(char *name, U_CHAR *bmpfileheader, U_CHAR *bmpinfoheader, U_CHAR *color_table, U_CHAR *data)
{
   FILE *input_file = 0 ;
   
   INT32 FileSize = 0 ;
   INT32 bfOffBits =0 ;
   INT32 headerSize =0 ;
   INT32 biWidth = 0 ;		
   INT32 biHeight = 0 ;
   INT16 biPlanes = 0 ;
   INT16 BitCount = 0 ;
   INT32 biCompression = 0 ;
   INT32 biImageSize = 0;
   INT32 biXPelsPerMeter = 0 ,biYPelsPerMeter = 0 ;
   INT32 biClrUsed = 0 ;
   INT32 biClrImp = 0 ;

   /* }ɮ */
   if( ( input_file = fopen(name,"rb") ) == NULL ){
      fprintf(stderr,"File can't open.\n");
      exit(0);
   }

   FREAD(input_file,bmpfileheader,14);
   FREAD(input_file,bmpinfoheader,40);

   if (GET_2B(bmpfileheader,0) == 0x4D42) /* 'BM' */
      fprintf(stdout,"BMP file.\n");
   else{
      fprintf(stdout,"Not bmp file.\n");
      exit(0);
   }

   FileSize           =   GET_4B(bmpfileheader,2);
   bfOffBits         =   GET_4B(bmpfileheader,10);
   headerSize      =   GET_4B(bmpinfoheader,0);
   biWidth           =   GET_4B(bmpinfoheader,4);
   biHeight          =   GET_4B(bmpinfoheader,8);
   biPlanes          =   GET_2B(bmpinfoheader,12);
   BitCount          =   GET_2B(bmpinfoheader,14);
   biCompression   =   GET_4B(bmpinfoheader,16);
   biImageSize      =   GET_4B(bmpinfoheader,20);
   biXPelsPerMeter =   GET_4B(bmpinfoheader,24);
   biYPelsPerMeter =   GET_4B(bmpinfoheader,28);
   biClrUsed         =   GET_4B(bmpinfoheader,32);
   biClrImp          =   GET_4B(bmpinfoheader,36);

   printf("FileSize = %ld \n"
	"DataOffset = %ld \n"
           "HeaderSize = %ld \n"
	"Width = %ld \n"
	"Height = %ld \n"
	"Planes = %d \n"
	"BitCount = %d \n"
	"Compression = %ld \n"
	"ImageSize = %ld \n"
	"XpixelsPerM = %ld \n"
	"YpixelsPerM = %ld \n"
	"ColorsUsed = %ld \n"
	"ColorsImportant = %ld \n",FileSize,bfOffBits,headerSize,biWidth,biHeight,biPlanes,
	BitCount,biCompression,biImageSize,biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImp);

   if (BitCount != 8) {
      fprintf(stderr,"Not a 8-bit file.\n");
      fclose (input_file);
      exit(0);
   }

   FREAD(input_file,color_table,1024);

   //
   
   //
   fseek(input_file,bfOffBits,SEEK_SET);
   FREAD(input_file,data,((biWidth*1 +3)/4 *4)*biHeight*1);
   // 
   fclose (input_file);
}

//void padding(